package com.abcinc.phonelist;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.struts.action.Action;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

/**
 * Implementation of <strong>Action</strong> that saves a contact.
 *
 * @author Wellie W. Chao
 */
public class SaveAction extends Action {

	// ------------------------------------------------------- Static Variables

	// ----------------------------------------------------- Instance Variables

	// ----------------------------------------------------------- Constructors

	// -------------------------------------------------------- Private Methods

	// ------------------------------------------------- Private Static Methods

	// ----------------------------------------------------- Accessors/Mutators

	// --------------------------------------------------------- Public Methods

	/**
	 * Process the specified HTTP request, and create the corresponding HTTP
	 * response (or forward to another web component that will create it).
	 * Return an <code>ActionForward</code> instance describing where and how
	 * control should be forwarded, or <code>null</code> if the response has
	 * already been completed.
	 *
	 * @param mapping The ActionMapping used to select this instance
	 * @param form The optional ActionForm bean for this request (if any)
	 * @param request The HTTP request we are processing
	 * @param response The HTTP response we are creating
	 *
	 * @exception Exception if the application business logic throws an
	 * exception
	 */
	public ActionForward execute
		(ActionMapping mapping,
		 ActionForm form,
		 HttpServletRequest request,
		 HttpServletResponse response)
		throws Exception {

		// Was this transaction cancelled?
		if (isCancelled(request)) {
			// Remove the obsolete form bean
			removeFormBean(mapping, request);
			// get the list of contacts in memory
			List contactBeanList = ContactDatabase.getList();
			// put displayed information in request attribute context for the JSP
			request.setAttribute("contactBeanList", contactBeanList);
			return (mapping.findForward("success"));
		}

		EditForm editForm = (EditForm) form;
		ActionErrors errors = new ActionErrors();

		// check for errors here.
		//if (condition) { errors.add(...); }

		// Report any errors we have discovered back to the original form
		if (!errors.isEmpty()) {
			saveErrors(request, errors);
			return mapping.getInputForward();
		}

		// figure out whether the form was in new or edit mode.
		Integer contactId = editForm.getId();
		boolean isNew = (contactId == null);
		ContactBean contactBean = null;
		if (isNew) {
			// create a new contact chosen as action, so get a new ID value
			// and instantiate a new ContactBean object.
			contactId = Sequence.getId();
			contactBean = new ContactBean();
			contactBean.setId(contactId);
		} else {
			// edit existing contact chosen as action, so retrieve the
			// existing contact from the database.
			contactBean = ContactDatabase.getBeanById(editForm.getId());
		}

		// copy values from the form to the ContactBean.
		contactBean.setLastName(editForm.getLastName());
		contactBean.setFirstName(editForm.getFirstName());
		contactBean.setPhone(editForm.getPhone());
		contactBean.setFax(editForm.getFax());
		contactBean.setEmail(editForm.getEmail());

		// if the user chose to create a new contact, add the new contact
		// to the database. If the user chose to edit an existing contact,
		// it is unnecessary to do anything further because we just
		// modified the ContactBean object in the database directly. With
		// a relational database, you would want to perform an update for
		// existing rows.
		if (isNew) {
			ContactDatabase.addBean(contactBean);
		}

		// Remove the obsolete form bean
		removeFormBean(mapping, request);

		// get the list of contacts in memory
		List contactBeanList = ContactDatabase.getList();

		// put displayed information in request attribute context for the JSP
		request.setAttribute("contactBeanList", contactBeanList);

		// Forward control to the specified success URI
		return (mapping.findForward("success"));

	}

	/**
	 * Remove the obsolete form bean for the current action.
	 *
	 * @param mapping the ActionMapping instance for the current
	 * application.
	 * @param request the request object for the current HTTP request.
	 */
	private void removeFormBean(ActionMapping mapping, HttpServletRequest request) {
		if (mapping.getAttribute() != null) {
			if ("request".equals(mapping.getScope())) {
				request.removeAttribute(mapping.getAttribute());
			} else {
				HttpSession session = request.getSession();
				session.removeAttribute(mapping.getAttribute());
			}
		}
	}

}
